"""
Site index.

This module is responsible for loading site content at request time, using a
Whoosh document index.
"""

import os
import json
from typing import List, Iterable
from ..domain import IndexablePage, SearchResults, SearchResult, Page, \
    Component, Schema
from arxiv.base.globals import get_application_config

from whoosh import fields, index
from whoosh.qparser import QueryParser
from whoosh.system import emptybytes

from arxiv.base import logging


logger = logging.getLogger(__name__)


class JSON(fields.TEXT):
    def to_bytes(self, value: dict) -> bytes:
        return super(JSON, self).to_bytes(json.dumps(value))

    def from_bytes(self, bs: bytes) -> dict:
        return json.loads(super(JSON, self).from_bytes(bs))

    def index(self, value: dict, **kwargs) -> Iterable:
        yield (self.to_bytes(value), 1, 1.0, emptybytes)
        return


SCHEMA = fields.Schema(
    title=fields.TEXT(stored=True),
    path=fields.ID(stored=True),
    parents=fields.IDLIST(stored=True),
    slug=fields.ID(stored=True),
    content=fields.TEXT(stored=True),
    content_path=fields.TEXT(stored=True),
    template=fields.TEXT(stored=True)
)

COMPONENT_SCHEMA = fields.Schema(
    title=fields.TEXT(stored=True),
    path=fields.ID(stored=True),
    data=JSON(stored=True),
    slug=fields.ID(stored=True),
    content=fields.TEXT(stored=True),
    content_path=fields.TEXT(stored=True)
)

SCHEMA_SCHEMA = fields.Schema(
    title=fields.TEXT(stored=True),
    path=fields.ID(stored=True),
    data=JSON(stored=True),
    slug=fields.ID(stored=True),
    content_path=fields.TEXT(stored=True)
)


class PageDoesNotExist(Exception):
    """A requested :class:`Page` does not exist."""


class ComponentDoesNotExist(Exception):
    """A requested :class:`Component` does not exist."""


class SchemaDoesNotExist(Exception):
    """A requested :class:`Schema` does not exist."""


class DuplicateSlug(Exception):
    """Two pages with the same slug were encountered!."""


def create_index() -> index.Index:
    """Initialize the index."""
    index_path = _get_index_path()
    components_index_path = _get_component_index_path()
    schema_index_path = _get_schema_index_path()
    for path in [index_path, components_index_path, schema_index_path]:
        if not os.path.exists(path):
            os.mkdir(path)
    index.create_in(index_path, SCHEMA)
    index.create_in(components_index_path, COMPONENT_SCHEMA)
    index.create_in(schema_index_path, SCHEMA_SCHEMA)


def add_components(components: Iterable[Component]) -> None:
    idx = _get_index(_get_component_index_path())
    writer = idx.writer()
    for component in components:
        writer.add_document(
            title=component.title,
            path=component.path,
            slug=component.slug,
            content=component.content,
            content_path=component.content_path,
            data=component.data
        )
        logger.debug('Added component %s', component.path)
    writer.commit()


def add_schemas(schemas: Iterable[Schema]) -> None:
    idx = _get_index(_get_schema_index_path())
    writer = idx.writer()
    for schema in schemas:
        writer.add_document(
            title=schema.title,
            path=schema.path,
            slug=schema.slug,
            content_path=schema.content_path,
            data=schema.data
        )
        logger.debug('Added schema %s', schema.path)
    writer.commit()


def add_documents(pages: Iterable[IndexablePage]) -> None:
    """
    Add :class:`.IndexablePage`s to the search index.

    Parameters
    ----------
    pages : iterable
        An iterable that yields :class:`.IndexablePage` tuples.

    Raises
    ------
    :class:`DuplicateSlug`
        Raised if the same slug is encountered twice.

    """
    idx = _get_index(_get_index_path())
    writer = idx.writer()
    slugs = set()
    for page, content in pages:
        # Slugs must be unique, so we want to complain as early as possible
        # if that does not appear to be the case.
        if page.slug in slugs:
            raise DuplicateSlug(
                'More than one page has slug `%s`; check the page at %s'
                % (page.slug, page.path)
            )
        slugs.add(page.slug)

        writer.add_document(
            title=page.title,
            path=page.path,
            slug=page.slug,
            content=content,
            content_path=page.content_path,
            parents=page.parent_paths,
            template=page.template
        )
        logger.debug('Added page %s', page.path)
    writer.commit()


def get_by_path(path: str, get_parents: bool = True,
                get_children: bool = True) -> Page:
    """
    Load a :class:`.Page` by its path.

    Parameters
    ----------
    path : str
        The relative path of the page.
    get_parents : bool
        Whether or not to load the parents of this page.
    get_children : bool
        Whether or not to load the children of this page.

    Returns
    -------
    :class:`.Page`
    """
    idx = _get_index(_get_index_path())
    logger.debug('Searching with index %s', idx)
    with idx.searcher() as searcher:
        logger.debug('Searching with path=%s', path)
        result = searcher.document(path=path)
    if result is None:
        logger.debug('Result is None')
        raise PageDoesNotExist('No such page')
    return Page(
        title=result['title'],
        path=result['path'],
        content_path=result['content_path'],
        template=result.get('template'),
        slug=result['slug'],
        parents=_get_parents(result) if get_parents else None,
        children=_get_children(result) if get_children else None,
    )


def get_component_by_path(path: str) -> Component:
    """
    Load a :class:`.Component` by its path.

    Parameters
    ----------
    path : str
        The relative path of the component.

    Returns
    -------
    :class:`.Component`
    """
    idx = _get_index(_get_component_index_path())
    with idx.searcher() as searcher:
        result = searcher.document(path=path)
    if result is None:
        raise ComponentDoesNotExist('No such component')
    return Component(
        title=result['title'],
        path=result['path'],
        content_path=result['content_path'],
        slug=result['slug'],
        data=result['data']
    )


def get_schema_by_path(path: str) -> Schema:
    """
    Load a :class:`.Schema` by its path.

    Parameters
    ----------
    path : str
        The relative path of the schema.

    Returns
    -------
    :class:`.Schema`
    """
    idx = _get_index(_get_schema_index_path())
    with idx.searcher() as searcher:
        result = searcher.document(path=path)
    if result is None:
        raise SchemaDoesNotExist('No such schema')
    return Schema(
        title=result['title'],
        path=result['path'],
        content_path=result['content_path'],
        slug=result['slug'],
        data=result['data']
    )


def get_by_slug(slug: str, get_parents: bool = True,
                get_children: bool = True) -> SearchResult:
    """
    Load a :class:`.Page` by its unique slug.

    Parameters
    ----------
    path : str
        The relative path of the page.
    get_parents : bool
        Whether or not to load the parents of this page.
    get_children : bool
        Whether or not to load the children of this page.

    Returns
    -------
    :class:`.Page`
    """
    idx = _get_index(_get_index_path())
    with idx.searcher() as searcher:
        result = searcher.document(slug=slug)
    if result is None:
        raise PageDoesNotExist('No such page')

    return Page(
        title=result['title'],
        path=result['path'],
        content_path=result['content_path'],
        slug=result['slug'],
        parents=_get_parents(result) if get_parents else None,
        children=_get_children(result) if get_children else None,
    )


def find(query: str) -> SearchResults:
    """
    Perform a simple text search against the site index.

    Parameters
    ----------
    query : str
        Free text with which to perform a full-text query.

    Returns
    -------
    :class:`.SearchResults`
    """
    qp = QueryParser("content", schema=SCHEMA)
    q = qp.parse(query)

    idx = _get_index(_get_index_path())
    with idx.searcher() as searcher:
        results = searcher.search(q)

        return SearchResults(
            total_hits=len(results),
            scored_hits=results.scored_length(),
            results=[SearchResult(
                Page(
                    title=result['title'],
                    path=result['path'],
                    content_path=result['content_path'],
                    slug=result['slug'],
                    parents=_get_parents(result),
                    children=_get_children(result),
                ),
                str(result.highlights('content'))
            ) for result in results]
        )


def _get_parents(result: dict) -> List[Page]:
    return [get_by_path('/'.join(result['parents'][:i+1]), False, False)
            for i in range(len(result['parents']))]


def _get_children(result: dict) -> List[Page]:
    idx = _get_index(_get_index_path())
    with idx.searcher() as searcher:
        results = searcher.documents(parents=result['path'])
        return [
            Page(
                title=result['title'],
                path=result['path'],
                content_path=result['content_path'],
                slug=result['slug'],
            ) for result in results
        ]


def _get_index_path() -> str:
    """Get the index path from the current application."""
    config = get_application_config()
    return config.get('INDEX_NAME', 'idx')


def _get_component_index_path() -> str:
    """Get the index path from the current application."""
    config = get_application_config()
    _path = config.get('INDEX_NAME', 'idx')
    return f'{_path}_components'


def _get_schema_index_path() -> str:
    """Get the schema index path from the current application."""
    config = get_application_config()
    _path = config.get('INDEX_NAME', 'idx')
    return f'{_path}_schemas'


def _get_index(index_path: str) -> index.Index:
    """Get the index in ``index_path``."""
    if not os.path.exists(index_path):
        raise ValueError(f"{index_path} does not exist")
    return index.open_dir(index_path)



"""UI routes for the documents service."""

import os
from typing import Callable, Union, Dict, Mapping
from flask import Blueprint, render_template, Markup, request, url_for, Flask
from werkzeug.exceptions import HTTPException, NotFound, InternalServerError

from arxiv.base import logging
from flask import render_template_string, current_app
import jinja2

from .services import index, site
from . import render
from .domain import Page, Component


logger = logging.getLogger(__name__)


docs = Blueprint('docs', __name__, url_prefix='/_docs',
                 static_folder='static',
                 template_folder='templates')


class ComponentLoader(object):
    """Recursive attr-loader for components."""

    def __init__(self, key: str) -> None:
        """Initialize a loader with a key (path partial)."""
        self._key = key

    def __getattr__(self, key: str) -> Union['ComponentLoader', Component]:
        """Attempt to load a component by path."""
        path = '/'.join([self._key, key]).lstrip('/')
        try:
            component = index.get_component_by_path(path)
        except index.ComponentDoesNotExist:
            logger.debug(f'Could not load component at {path}')
            return ComponentLoader(path)
        logger.debug(f'Loaded component at {path}')
        return component


class SchemaLoader(object):
    """Recursive attr-loader for schemas."""

    def __init__(self, key: str) -> None:
        """Initialize a loader with a key (path partial)."""
        self._key = key

    def __getattr__(self, key: str) -> Union['SchemaLoader', Component]:
        """Attempt to load a component by path."""
        path = '/'.join([self._key, key]).lstrip('/')
        try:
            schema = index.get_schema_by_path(path)
        except index.SchemaDoesNotExist:
            logger.debug(f'Could not load schema at {path}')
            return SchemaLoader(path)
        logger.debug(f'Loaded schema at {path}')
        return schema


def inject_components() -> Dict[str, ComponentLoader]:
    """Add a root-level component loader to the rendering context."""
    return {'components': ComponentLoader('')}


def inject_schemas() -> Dict[str, SchemaLoader]:
    """Add a root-level schema loader to the rendering context."""
    return {'schemas': SchemaLoader('')}


def get_blueprint(app: Flask) -> Blueprint:
    """Generate a blueprint for the site based on the app config."""
    site_name = app.config["SITE_NAME"]
    static_url_path = f'{app.static_url_path}/{site_name}'
    blueprint = Blueprint(site_name, __name__, url_prefix='',
                          static_folder=app.config['STATIC_ROOT'],
                          template_folder=app.config['TEMPLATE_ROOT'],
                          static_url_path=static_url_path)

    def get_link_dereferencer(page: Page) -> Callable:
        """Generate a dereferencer function for paths relative to a page."""
        def link_dereferencer(href: str) -> str:
            """Link dereferencer for use during HTML rendering."""
            if not href or href.startswith('http'):
                logger.debug('not an internal link: %s', href)
                return href
            try:
                path = index.get_by_slug(href).path
                logger.debug('got path by slug: %s', href)
            except index.PageDoesNotExist:
                abspath = '/'.join([page.path, href])
                try:
                    path = index.get_by_path(abspath).path
                    logger.debug('got path by abspath: %s', href)
                except index.PageDoesNotExist:
                    logger.debug('could not dereference path: %s', href)
                    return href
            return url_for(f'{site_name}.from_sitemap', path=path)
        return link_dereferencer

    def get_static_dereferencer(page: Page) -> Callable:
        """Generate a dereferencer for static files relative to a page."""
        def static_deferencer(src: str) -> str:
            """Static URL dereferencer for use during HTML rendering."""
            if not src or src.startswith('http'):
                return src
            if page.is_index_page:
                path_parts = page.path.split('/')
            else:
                path_parts = page.path.split('/')[:-1]
            filename = "/".join(path_parts + [src])
            return url_for(f'{site_name}.static', filename=filename)
        return static_deferencer

    @blueprint.route('/search', methods=['GET'])
    def search():
        """Handle a search request."""
        q = request.args.get('q')
        results = index.find(q)
        try:
            return render_template(f'{site_name}/search.html',
                                   results=results,
                                   q=q,
                                   site_name=site_name)
        except jinja2.exceptions.TemplateNotFound:
            return render_template('docs/search.html', results=results, q=q)

    @blueprint.route('/', defaults={'path': ''})
    @blueprint.route('/<path:path>')
    def from_sitemap(path: str):
        """Route the request dynamically, based on the indexed site map."""
        logger.debug('Request for path: %s', path)
        try:
            page = index.get_by_path(path)
        except index.PageDoesNotExist as e:
            logger.debug('Page does not exist')
            raise NotFound('Page does not exist') from e

        try:
            content = site.content(page)
        except site.PageLoadFailed as e:
            raise InternalServerError('Could not load page') from e

        # We'll re-use this as part of the final template context, in case we
        # need it elsewhere.
        def _render_markdown(s: str) -> Markup:
            return Markup(render.render(s, get_link_dereferencer(page),
                                        get_static_dereferencer(page)))

        context = {
            'page': page,
            'pagetitle': page.title,
            'render_markdown': _render_markdown,
            'site_name': site_name
        }
        rendered = render_template_string(_render_markdown(content), **context)

        context.update({'content': Markup(rendered)})
        if page.template is not None:
            template = page.template
        else:
            template = f'{site_name}/page.html'

        try:
            return render_template(template, **context)
        except jinja2.exceptions.TemplateNotFound:
            return render_template('docs/page.html', **context)

    blueprint.context_processor(inject_components)
    blueprint.context_processor(inject_schemas)
    return blueprint



"""Domain classes for arxiv-docs service."""

from typing import Any, Optional, Type, NamedTuple, List, Tuple
import bleach
import markdown


class Page(NamedTuple):
    title: str
    path: str
    content_path: str
    slug: str
    parents: Optional[List] = None
    children: Optional[List] = None
    content: Optional[str] = None
    template: Optional[str] = None

    @property
    def parent_paths(self) -> Optional[List[str]]:
        if self.parents is None:
            return None
        return [parent for parent in self.parents]

    @property
    def child_paths(self) -> Optional[List[str]]:
        if self.children is None:
            return None
        return [child.path for child in self.children]

    @property
    def is_index_page(self):
        return self.content_path.endswith('/index.md')


class SearchResult(NamedTuple):
    page: Page
    highlights: str


class SearchResults(NamedTuple):
    total_hits: int
    scored_hits: int
    results: List[SearchResult]


class IndexablePage(NamedTuple):
    page: Page
    indexable_content: str


class Component(NamedTuple):
    title: str
    path: str
    content_path: str
    slug: str
    data: dict
    content: Optional[str] = None


class Schema(NamedTuple):
    title: str
    path: str
    content_path: str
    slug: str
    data: dict



"""Build a site."""

import os
import shutil
from typing import List

from .domain import IndexablePage, Page
from .services import site, index
from .render import render

import bleach


def index_site(site_path: str, site_name: str, root_path: str,
               static_root: str, template_folder: str) -> None:
    """Index the entire site."""
    print(f'Building site from {site_path}')
    index.create_index()
    print('Created index')

    index.add_components(site.load_components(site_path))
    print('Added components')

    index.add_schemas(site.load_schemas(site_path))
    print('Added schemas')

    index.add_documents((
        (page, bleach.clean(render(page.content), strip=True, tags=[]))
        for page in site.load_all(site_path)
    ))
    print('Added pages')

    # Copy static files into Flask's static directory. If we're deploying
    # to a CDN, this should happen first so that Flask knows what it's
    # working with.
    for path, source_path in site.load_static(site_path):
        target_path = os.path.abspath(os.path.join(static_root, path))

        # Make sure that the directory into which we're putting this static
        # file actually exists.
        target_dir, _ = os.path.split(target_path)
        if not os.path.exists(target_dir):
            os.makedirs(target_dir)

        # This will overwrite whatever is already there.
        print(f"Static: copy {source_path} to {target_path}")
        shutil.copy(os.path.abspath(source_path), target_path)
    print('Added static files')

    temp_root = os.path.join(root_path, os.path.split(template_folder)[0])
    print(f'Copy templates to {temp_root}')
    for path, source_path in site.load_templates(site_path):
        target_path = os.path.join(temp_root, site_name, path)
        print(temp_root, site_name, path, target_path)

        # Make sure that the directory into which we're putting this
        # template actually exists.
        target_dir, _ = os.path.split(target_path)
        if not os.path.exists(target_dir):
            os.makedirs(target_dir)
        print(path, target_path)
        # This will overwrite whatever is already there.
        shutil.copy(os.path.abspath(source_path), target_path)
    print('Added templates')



"""Responsible for rendering markdown content to HTML."""

from typing import Callable, Optional, Mapping, Union
import xml.etree.ElementTree as ET
from markdown import markdown, Markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

from arxiv.base import logging

logger = logging.getLogger(__name__)


def render(content: str, dereferencer: Optional[Callable] = None,
           static_dereferencer: Optional[Callable] = None) -> str:
    """
    Render markdown content to HTML.

    Parameters
    ----------
    content : str
        Markdown content.
    dereferencer : function
        Used for generating URLs from internal paths and slugs. Should accept
        a HREF value (str), and return a URL (str). Optional.
    static_dereferencer : function
        Used for generating URLs for static files. Should accept
        a src value (str), and return a URL (str). Optional.

    Returns
    -------
    str
        Rendered HTML.

    """
    extensions = ['tables', 'fenced_code', 'codehilite']
    if dereferencer is not None:
        extensions.append(ReferenceExtension('a', 'href', dereferencer))
    if static_dereferencer is not None:
        extensions.append(
            ReferenceExtension('img', 'src', static_dereferencer)
        )
    return markdown(content, extensions=extensions)


class ReferenceProcessor(Treeprocessor):
    """Convert internal links to full paths."""

    def __init__(self, tag: str, attr: str,
                 dereferencer: Optional[Callable] = None) -> None:
        """Set the link dereferencer for use during processing."""
        self.dereferencer = dereferencer
        self.tag = tag
        self.attr = attr

    def run(self, root: ET.ElementTree) -> None:
        """Perform link conversion on ``root``."""
        if self.dereferencer is not None:
            self.translate_anchors(root)

    def translate_anchors(self, element: ET.Element) -> None:
        """Traverse ``element`` looking for and updating anchor elements."""
        for child in element:
            if child.tag == self.tag:
                value = child.attrib.get(self.attr)
                try:
                    child.attrib[self.attr] = self.dereferencer(value)
                except KeyError:
                    continue
            self.translate_anchors(child)


class ReferenceExtension(Extension):
    """Adds :class:`.ReferenceProcessor` to the markdown processor."""

    def __init__(self, tag: str, attr: str, dereferencer: Optional[Callable]) \
            -> None:
        """Set the link dereferencer for use during processing."""
        self.tag = tag
        self.attr = attr
        self.dereferencer = dereferencer

    def extendMarkdown(self, md: Markdown, md_globals: Mapping) -> None:
        """Add :class:`.ReferenceProcessor` to the markdown processor."""
        inst = ReferenceProcessor(self.tag, self.attr, self.dereferencer)
        md.treeprocessors[f'{self.tag}_{self.attr}_reference_processor'] = inst



"""Application factory for references service components."""

import logging

from flask import Flask

from arxiv.base import Base
from . import routes


def create_web_app() -> Flask:
    """Initialize an instance of the static pages application."""
    from . import config

    app = Flask(config.SITE_NAME)
    app.config.from_object(config)

    Base(app)

    app.register_blueprint(routes.docs)
    # app.register_blueprint(routes.blueprint)
    app.register_blueprint(routes.get_blueprint(app))
    return app



import os

SECRET_KEY = os.environ.get('SECRET_KEY', 'asdf1234')
INDEX_NAME = os.environ.get('INDEX_NAME', 'idx')
SITE_NAME = os.environ.get('SITE_NAME', 'site')
SITE_PATH = os.path.join(
    os.environ.get('BUILD_PATH', ''),
    SITE_NAME
)
SERVER_NAME = os.environ.get('DOCS_SERVER_NAME')
ARXIV_URLS = {}
LOGLEVEL = os.environ.get('LOGLEVEL', 10)


VERSION = os.environ.get('VERSION')
SOURCE = os.environ.get('SOURCE')
BUILD_TIME = os.environ.get('BUILD_TIME')

STATIC_ROOT = os.environ.get('STATIC_ROOT', 'static')
TEMPLATE_ROOT = os.environ.get('TEMPLATE_ROOT', 'templates')



"""
Site loader.

This module is responsible for loading site content at build time.
"""

import os
import yaml
from collections import defaultdict
from typing import List, Tuple, Dict, Callable, Iterable
from ..domain import Page, Component, Schema

import frontmatter
from flask import current_app

from prance import ResolvingParser
from slugify import slugify


class PageLoadFailed(Exception):
    """Could not load the content of a page."""


def content(page: Page) -> str:
    try:
        return frontmatter.load(page.content_path).content
    except IOError as e:
        raise PageLoadFailed('Failed to load page') from e


def load_all(site_path: str) -> Iterable[Page]:
    pages_path = os.path.join(site_path, 'pages')

    for dirpath, dirnames, filenames in os.walk(pages_path):
        for filename in filenames:
            if filename.endswith('.md'):
                yield _load_page(pages_path, dirpath, filename)


def load_components(site_path: str) -> Iterable[Component]:
    components_path = os.path.join(site_path, 'components')
    for dirpath, dirnames, filenames in os.walk(components_path):
        for filename in filenames:
            try:
                _, ext = filename.rsplit('.', 1)
            except ValueError:
                continue
            if ext in ['md', 'yaml', 'yml']:
                yield _load_component(components_path, dirpath, filename)


def load_schemas(site_path: str) -> Iterable[Schema]:
    schemas_path = os.path.join(site_path, 'schemas')
    for dirpath, dirnames, filenames in os.walk(schemas_path):
        for filename in filenames:
            try:
                name, ext = filename.rsplit('.', 1)
            except ValueError:
                continue
            if ext in ['yaml', 'yml']:
                content_path = os.path.join(dirpath, filename)
                _, parent = os.path.split(dirpath)
                path = os.path.join(parent, name)
                try:
                    spec = ResolvingParser(
                        content_path,
                        backend='openapi-spec-validator'
                    ).specification
                except Exception as e:
                    continue
                yield Schema(
                    title=spec.get('title', filename),
                    path=path,
                    content_path=content_path,
                    slug=slugify(filename),
                    data=spec
                )


def load_static(site_path: str) -> List[Tuple[str, str]]:
    files = []
    pages_path = os.path.join(site_path, 'pages')
    for dirpath, dirnames, filenames in os.walk(pages_path):
        for filename in filenames:
            if filename.endswith('.md') or filename.startswith('.'):
                continue
            content_path = os.path.join(dirpath, filename)
            path = content_path[len(pages_path):].strip('/')
            files.append((path, content_path))
    return files


def load_templates(site_path: str) -> List[Tuple[str, str]]:
    files = []
    pages_path = os.path.join(site_path, 'templates')
    for dirpath, dirnames, filenames in os.walk(pages_path):
        for filename in filenames:
            if not filename.endswith('.html') or filename.startswith('.'):
                continue
            content_path = os.path.join(dirpath, filename)
            path = content_path[len(pages_path):].strip('/')
            files.append((path, content_path))
    return files


def _load_page(rootpath: str, dirpath: str, filename: str) -> Page:
    content_path = os.path.join(dirpath, filename)
    path = content_path[len(rootpath):].strip('/')
    if path.endswith('.md'):
        path = path[:-3]
    if path.split('/')[-1] == 'index':
        path = '/'.join(path.split('/')[:-1])

    page_data = frontmatter.load(content_path)
    parents = path.split('/')[:-1]
    title = _get_title(page_data, filename)
    template = page_data.get('template')

    site_name = current_app.config['SITE_NAME']
    if template is not None:
        template = f'{site_name}/{template}'
    page = Page(
        title=title,
        path=path,
        content_path=content_path,
        slug=page_data.get('slug', slugify(path)),
        content=page_data.content,
        parents=parents,
        template=template
    )
    return page


def _load_markdown_component(content_path: str, filename: str, path: str) \
        -> Component:
    page_data = frontmatter.load(content_path)
    return Component(
        title=_get_title(page_data, filename),
        path=path,
        content_path=content_path,
        slug=page_data.get('slug', slugify(path)),
        content=page_data.content,
        data=page_data.to_dict()
    )


def _load_yaml_component(content_path: str, filename: str, path: str) \
        -> Component:
    with open(content_path) as f:
        page_data = yaml.load(f)
    return Component(
        title=path,
        path=path,
        content_path=content_path,
        slug=page_data.get('slug', slugify(path)),
        data=page_data
    )


def _load_component(rootpath: str, dirpath: str, filename: str) -> Component:
    content_path = os.path.join(dirpath, filename)
    path = content_path[len(rootpath):].strip('/')
    if path.endswith('.md') or path.endswith('.yml') or path.endswith('.yaml'):
        path, ext = path.rsplit('.', 1)

    if filename.endswith('.md'):
        return _load_markdown_component(content_path, filename, path)

    elif filename.endswith('.yml') or filename.endswith('.yaml'):
        return _load_yaml_component(content_path, filename, path)


def _get_title(page_data: dict, filename: str) -> str:
    title = page_data.get('title')
    if title is None:
        for line in page_data.content.split('\n'):
            cleaned = line.replace('#', '').strip()
            if cleaned:
                title = cleaned
                break
    if title is None:
        title = filename.rstrip('.md')
    return title



"""Install arXiv Docs package."""

from setuptools import setup, find_packages

setup(
    name='arxiv-docs',
    version='0.1.1',
    packages=[f'arxiv.{package}' for package
              in find_packages('./arxiv', exclude=['*test*'])],
    install_requires=[
        "flask",
        "markdown",
        "pyyaml",
        "whoosh",
        "arxiv-base",
        "bleach",
        "uwsgi",
        "python-frontmatter",
        "python-slugify",
        "Pygments"
    ],
    zip_safe=False,
    include_package_data=True
)
